@@ -396,7 +396,7 @@ AFSClose( IN PDEVICE_OBJECT LibDeviceObject,
 
                         AFSDbgLogMsg( AFS_SUBSYSTEM_CLEANUP_PROCESSING,
                                       AFS_TRACE_LEVEL_VERBOSE,
-                                      "AFSClose Deleting dir etry %08lX (%08lX) for %wZ\n",
+                                      "AFSClose Deleting dir entry %08lX (%08lX) for %wZ\n",
                                       pDirCB,
                                       pObjectInfo,
                                       &pDirCB->NameInformation.FileName);
