@@ -1114,13 +1114,17 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
 
                                     AFSDeleteObjectInfo( pCurrentObject->Specific.Directory.PIOCtlDirectoryCB->ObjectInformation);
 
-                                    ExFreePool( pCurrentObject->Specific.Directory.PIOCtlDirectoryCB);
+                                    ExDeleteResourceLite( &pCurrentChildObject->Specific.Directory.PIOCtlDirectoryCB->NonPaged->Lock);
+
+                                    AFSExFreePool( pCurrentChildObject->Specific.Directory.PIOCtlDirectoryCB->NonPaged);
+
+                                    AFSExFreePool( pCurrentObject->Specific.Directory.PIOCtlDirectoryCB);
                                 }
 
                                 AFSDbgLogMsg( AFS_SUBSYSTEM_CLEANUP_PROCESSING,
                                               AFS_TRACE_LEVEL_VERBOSE,
                                               "AFSPrimaryWorker Deleting deleted object %08lX\n",
-                                                                    pCurrentObject);
+                                              pCurrentObject);
 
                                 AFSDeleteObjectInfo( pCurrentObject);
 
@@ -1275,8 +1279,8 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
                                     AFSDbgLogMsg( AFS_SUBSYSTEM_CLEANUP_PROCESSING,
                                                   AFS_TRACE_LEVEL_VERBOSE,
                                                   "AFSPrimaryWorker Deleting DE %wZ Object %08lX\n",
-                                                                    &pCurrentDirEntry->NameInformation.FileName,
-                                                                    pCurrentChildObject);
+                                                  &pCurrentDirEntry->NameInformation.FileName,
+                                                  pCurrentChildObject);
 
                                     AFSDeleteDirEntry( pCurrentObject,
                                                        pCurrentDirEntry);
@@ -1311,15 +1315,15 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
 
                                             ExDeleteResourceLite( &pCurrentChildObject->Specific.Directory.PIOCtlDirectoryCB->NonPaged->Lock);
 
-                                            ExFreePool( pCurrentChildObject->Specific.Directory.PIOCtlDirectoryCB->NonPaged);
+                                            AFSExFreePool( pCurrentChildObject->Specific.Directory.PIOCtlDirectoryCB->NonPaged);
 
-                                            ExFreePool( pCurrentChildObject->Specific.Directory.PIOCtlDirectoryCB);
+                                            AFSExFreePool( pCurrentChildObject->Specific.Directory.PIOCtlDirectoryCB);
                                         }
 
                                         AFSDbgLogMsg( AFS_SUBSYSTEM_CLEANUP_PROCESSING,
                                                       AFS_TRACE_LEVEL_VERBOSE,
                                                       "AFSPrimaryWorker Deleting object %08lX\n",
-                                                                            pCurrentChildObject);
+                                                      pCurrentChildObject);
 
                                         AFSDeleteObjectInfo( pCurrentChildObject);
                                     }
@@ -1342,10 +1346,10 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
                                 AFSDbgLogMsg( AFS_SUBSYSTEM_DIR_NODE_COUNT,
                                               AFS_TRACE_LEVEL_VERBOSE,
                                               "AFSPrimaryWorker Reset count to 0 on parent FID %08lX-%08lX-%08lX-%08lX\n",
-                                                  pCurrentObject->FileId.Cell,
-                                                  pCurrentObject->FileId.Volume,
-                                                  pCurrentObject->FileId.Vnode,
-                                                  pCurrentObject->FileId.Unique);
+                                              pCurrentObject->FileId.Cell,
+                                              pCurrentObject->FileId.Volume,
+                                              pCurrentObject->FileId.Vnode,
+                                              pCurrentObject->FileId.Unique);
 
 
                                 //
@@ -2071,7 +2075,7 @@ try_exit:
 
                 InterlockedDecrement( &Fcb->OpenReferenceCount);
 
-                ExFreePool( pWorkItem);
+                ExFreePoolWithTag( pWorkItem, AFS_WORK_ITEM_TAG);
             }
 
             AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
@@ -2155,7 +2159,7 @@ try_exit:
             if( pWorkItem != NULL)
             {
 
-                ExFreePool( pWorkItem);
+                ExFreePoolWithTag( pWorkItem, AFS_WORK_ITEM_TAG);
             }
 
             AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
@@ -2239,7 +2243,7 @@ try_exit:
             if( pWorkItem != NULL)
             {
 
-                ExFreePool( pWorkItem);
+                ExFreePoolWithTag( pWorkItem, AFS_WORK_ITEM_TAG);
             }
 
             AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
@@ -2308,7 +2312,7 @@ try_exit:
             if( pWorkItem != NULL)
             {
 
-                ExFreePool( pWorkItem);
+                ExFreePoolWithTag( pWorkItem, AFS_WORK_ITEM_TAG);
             }
 
             AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
@@ -2417,7 +2421,7 @@ try_exit:
             if( pWorkItem != NULL)
             {
 
-                ExFreePool( pWorkItem);
+                ExFreePoolWithTag( pWorkItem, AFS_WORK_ITEM_TAG);
             }
         }
     }
