@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -10,15 +10,12 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <roken.h>
 
 #include <afs/stds.h>
 
 #include <windows.h>
 #include <rpc.h>
-#include <errno.h>
-#include <stdlib.h>
-#include <stdio.h>		/* sprintf */
-#include <malloc.h>
 #include <afs/kautils.h>
 #include <afs/cm.h>
 #include <afs/cm_config.h>
@@ -26,7 +23,7 @@
 #include <afs/krb_prot.h>
 #include <rx/rxkad.h>
 #include <crypt.h>
-#include <des.h>
+#include <hcrypto/des.h>
 
 int krb_add_host(struct sockaddr_in *server_list_p);
 static void krb_set_port(long port);
@@ -109,13 +106,10 @@ ka_UserAuthenticateGeneral2(afs_int32 flags, char *name, char *instance,
     code = cm_SearchCellRegistry(1, realm, fullRealm, NULL, ka_AddHostProc, NULL);
     if (code && code != CM_ERROR_FORCE_DNS_LOOKUP)
         code = cm_SearchCellFile(realm, fullRealm, ka_AddHostProc, NULL);
-
-#ifdef AFS_AFSDB_ENV
     if (code) {
 	code =
 	    cm_SearchCellByDNS(realm, fullRealm, &ttl, ka_AddHostProc, NULL);
     }
-#endif
     if (code) {
 	*reasonP = "specified realm is unknown";
 	return (code);
@@ -126,7 +120,7 @@ ka_UserAuthenticateGeneral2(afs_int32 flags, char *name, char *instance,
 
     /* encrypt password, both ways */
     ka_StringToKey(password, upperRealm, &key1);
-    des_string_to_key(password, &key2);
+    DES_string_to_key(password, &key2);
 
     /* set port number */
     sp = getservbyname("kerberos", "udp");
@@ -184,7 +178,7 @@ ka_UserAuthenticateGeneral2(afs_int32 flags, char *name, char *instance,
 /*
  * krb_get_in_tkt()
  *
- * This code is descended from kerberos files krb_get_in_tkt.c and 
+ * This code is descended from kerberos files krb_get_in_tkt.c and
  * send_to_kdc.c, and one.c.
  */
 
@@ -209,7 +203,6 @@ static int krbONE = 1;
 #include <string.h>
 #include <time.h>
 
-#include <des.h>
 #include "krb.h"
 
 #include <sys/types.h>
@@ -292,7 +285,7 @@ KTEXT pkt_cipher(KTEXT packet);
  */
 
 /*
- * Check_response is a support routine for krb_get_in_tkt. 
+ * Check_response is a support routine for krb_get_in_tkt.
  *
  * Check the response with the supplied key. If the key is apparently
  * wrong, return INTK_BADPW, otherwise zero.
@@ -301,7 +294,7 @@ static
 check_response(KTEXT rpkt, KTEXT cip, char *service, char *instance,
 	       char *realm, struct ktc_encryptionKey *key)
 {
-    Key_schedule key_s;
+    DES_key_schedule key_s;
     char *ptr;
     char s_service[SNAME_SZ];
     char s_instance[INST_SZ];
@@ -316,9 +309,9 @@ check_response(KTEXT rpkt, KTEXT cip, char *service, char *instance,
     memcpy((char *)(cip->dat), (char *)pkt_cipher(rpkt), cip->length);
 
     /* decrypt ticket */
-    key_sched((char *)key, key_s);
-    pcbc_encrypt((C_Block *) cip->dat, (C_Block *) cip->dat,
-		 (long)cip->length, key_s, (des_cblock *) key, 0);
+    DES_key_sched((DES_cblock *)key, &key_s);
+    DES_pcbc_encrypt((DES_cblock *) cip->dat, (DES_cblock *) cip->dat,
+		     (long)cip->length, &key_s, (DES_cblock *) key, 0);
 
     /* Skip session key */
     ptr = (char *)cip->dat + 8;
@@ -430,7 +423,7 @@ krb_get_in_tkt_ext(user, instance, realm, service, sinstance, life, key1,
     KTEXT cip = &cip_st;	/* Returned Ciphertext */
     KTEXT_ST tkt_st;
     KTEXT tkt = &tkt_st;	/* Current ticket */
-    C_Block ses;		/* Session key for tkt */
+    DES_cblock ses;		/* Session key for tkt */
     int kvno;			/* Kvno for session key */
     unsigned char *v = pkt->dat;	/* Prot vers no */
     unsigned char *t = (pkt->dat + 1);	/* Prot msg type */
@@ -939,7 +932,7 @@ Andrew_StringToKey(str, cell, key)
 	    keybytes[i] = (unsigned char)(temp << 1);
 	}
     }
-    des_fixup_key_parity((unsigned char *)key);
+    DES_fixup_key_parity((DES_cblock *)key);
 }
 
 
@@ -949,7 +942,7 @@ StringToKey(str, cell, key)
      char *cell;		/* cell for password */
      struct ktc_encryptionKey *key;
 {
-    des_key_schedule schedule;
+    DES_key_schedule schedule;
     char temp_key[8];
     char ivec[8];
     char password[BUFSIZ];
@@ -963,14 +956,14 @@ StringToKey(str, cell, key)
 
     memcpy(ivec, "kerberos", 8);
     memcpy(temp_key, "kerberos", 8);
-    des_fixup_key_parity(temp_key);
-    des_key_sched(temp_key, schedule);
-    des_cbc_cksum(password, ivec, passlen, schedule, ivec);
+    DES_fixup_key_parity(temp_key);
+    DES_key_sched(temp_key, &schedule);
+    DES_cbc_cksum(password, ivec, passlen, &schedule, ivec);
 
     memcpy(temp_key, ivec, 8);
-    des_fixup_key_parity(temp_key);
-    des_key_sched(temp_key, schedule);
-    des_cbc_cksum(password, (char *)key, passlen, schedule, ivec);
+    DES_fixup_key_parity(temp_key);
+    DES_key_sched(temp_key, &schedule);
+    DES_cbc_cksum(password, (DES_cblock *)key, passlen, &schedule, ivec);
 
-    des_fixup_key_parity((char *)key);
+    DES_fixup_key_parity((DES_cblock *)key);
 }
