@@ -14,8 +14,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/libadmin/test/client.c,v 1.8.14.2 2008/10/27 23:41:46 shadow Exp $");
+#include <roken.h>
 
 #include "client.h"
 #include <afs/cellconfig.h>
@@ -34,23 +33,9 @@ RCSID
 #include <ubik.h>
 #include <ubik_int.h>
 #ifdef AFS_NT40_ENV
-#include <winsock2.h>
 #include <pthread.h>
 #endif
 
-/* These aren't coming from a header, currently, so they must stay here. 
-   Fix elsewhere, or leave alone. */
-extern int RXSTATS_RetrieveProcessRPCStats();
-extern int RXSTATS_RetrievePeerRPCStats();
-extern int RXSTATS_QueryProcessRPCStats();
-extern int RXSTATS_QueryPeerRPCStats();
-extern int RXSTATS_EnableProcessRPCStats();
-extern int RXSTATS_EnablePeerRPCStats();
-extern int RXSTATS_DisableProcessRPCStats();
-extern int RXSTATS_DisablePeerRPCStats();
-extern int RXSTATS_ClearProcessRPCStats();
-extern int RXSTATS_ClearPeerRPCStats();
-
 /*
  * This structure stores the client and server function lists.
  * This is kept separate from the actual interface definitions
@@ -456,7 +441,7 @@ GetStatSourceFromString(const char *type, afs_stat_source_t * src, int *port)
 
 typedef enum {
     AFS_PEER_STATS,
-    AFS_PROCESS_STATS
+    afs_proc_tESS_STATS
 } afs_stat_type_t, *afs_stat_type_p;
 
 static afs_stat_type_t
@@ -467,7 +452,7 @@ GetStatTypeFromString(const char *type)
     if (!strcmp(type, "peer")) {
 	rc = AFS_PEER_STATS;
     } else if (!strcmp(type, "process")) {
-	rc = AFS_PROCESS_STATS;
+	rc = afs_proc_tESS_STATS;
     } else {
 	ERR_EXT("stat_type must be process or peer");
     }
@@ -483,7 +468,6 @@ DoClientRPCStatsStateGet(struct cmd_syndesc *as, void *arock)
     } DoClientRPCStatsStateGet_parm_t;
     afs_status_t st = 0;
     struct rx_connection *conn;
-    int servAddr = 0;
     afs_stat_source_t type;
     int srvrPort;
     int typeIsValid = 0;
@@ -541,7 +525,6 @@ DoClientRPCStatsStateEnable(struct cmd_syndesc *as, void *arock)
     typedef enum { SERVER, PROCESS, STAT_TYPE } DoClientRPCStatsEnable_parm_t;
     afs_status_t st = 0;
     struct rx_connection *conn;
-    int servAddr = 0;
     afs_stat_source_t type;
     int srvrPort;
     int typeIsValid = 0;
@@ -597,7 +580,6 @@ DoClientRPCStatsStateDisable(struct cmd_syndesc *as, void *arock)
     } DoClientRPCStatsDisable_parm_t;
     afs_status_t st = 0;
     struct rx_connection *conn;
-    int servAddr = 0;
     afs_stat_source_t type;
     int srvrPort;
     int typeIsValid = 0;
@@ -699,7 +681,6 @@ DoClientRPCStatsList(struct cmd_syndesc *as, void *arock)
     typedef enum { SERVER, PROCESS, STAT_TYPE } DoClientRPCStatsList_parm_t;
     afs_status_t st = 0;
     struct rx_connection *conn;
-    int servAddr = 0;
     afs_stat_source_t type;
     int srvrPort;
     int typeIsValid = 0;
@@ -830,7 +811,6 @@ DoClientRPCStatsClear(struct cmd_syndesc *as, void *arock)
     } DoClientRPCStatsClear_parm_t;
     afs_status_t st = 0;
     struct rx_connection *conn;
-    int servAddr = 0;
     afs_stat_source_t type;
     int srvrPort;
     int typeIsValid = 0;
@@ -997,7 +977,6 @@ DoClientRPCStatsVersionGet(struct cmd_syndesc *as, void *arock)
     afs_status_t st = 0;
     struct rx_connection *conn;
     afs_stat_source_t type;
-    int servAddr = 0;
     int srvrPort;
     int typeIsValid = 0;
     afs_RPCStatsVersion_t version;
@@ -1052,7 +1031,6 @@ DoClientCMGetServerPrefs(struct cmd_syndesc *as, void *arock)
     afs_status_t st = 0;
     typedef enum { SERVER, PORT } DoClientCMGetServerPrefs_parm_t;
     struct rx_connection *conn;
-    int servAddr = 0;
     int srvrPort = AFSCONF_CALLBACKPORT;
     afs_CMServerPref_t prefs;
     void *iter;
@@ -1122,7 +1100,6 @@ DoClientCMListCells(struct cmd_syndesc *as, void *arock)
     afs_status_t st = 0;
     typedef enum { SERVER, PORT } DoClientCMListCells_parm_t;
     struct rx_connection *conn;
-    int servAddr = 0;
     int srvrPort = AFSCONF_CALLBACKPORT;
     afs_CMListCell_t cellInfo;
     void *iter;
@@ -1177,7 +1154,6 @@ DoClientCMLocalCell(struct cmd_syndesc *as, void *arock)
     afs_status_t st = 0;
     typedef enum { SERVER, PORT } DoClientCMLocalCell_parm_t;
     struct rx_connection *conn;
-    int servAddr = 0;
     int srvrPort = AFSCONF_CALLBACKPORT;
     afs_CMCellName_t cellname;
 
@@ -1236,7 +1212,6 @@ DoClientCMClientConfig(struct cmd_syndesc *as, void *arock)
     afs_status_t st = 0;
     typedef enum { SERVER, PORT } DoClientCMLocalCell_parm_t;
     struct rx_connection *conn;
-    int servAddr = 0;
     int srvrPort = AFSCONF_CALLBACKPORT;
     afs_ClientConfig_t config;
 
