@@ -66,6 +66,9 @@
 #include "rx_internal.h"
 #include "rx_stats.h"
 
+#include "rx_conn.h"
+#include "rx_call.h"
+
 #ifdef RX_LOCKS_DB
 /* rxdb_fileID is used to identify the lock location, along with line#. */
 static int rxdb_fileID = RXDB_FILE_RX_PACKET;
@@ -1345,9 +1348,7 @@ rxi_AllocSendPacket(struct rx_call *call, int want)
 	 * just wait.  */
 	NETPRI;
 	call->flags |= RX_CALL_WAIT_PACKETS;
-        MUTEX_ENTER(&rx_refcnt_mutex);
 	CALL_HOLD(call, RX_CALL_REFCOUNT_PACKET);
-        MUTEX_EXIT(&rx_refcnt_mutex);
 	MUTEX_EXIT(&call->lock);
 	rx_waitingForPackets = 1;
 
@@ -1358,9 +1359,7 @@ rxi_AllocSendPacket(struct rx_call *call, int want)
 #endif
 	MUTEX_EXIT(&rx_freePktQ_lock);
 	MUTEX_ENTER(&call->lock);
-        MUTEX_ENTER(&rx_refcnt_mutex);
 	CALL_RELE(call, RX_CALL_REFCOUNT_PACKET);
-        MUTEX_EXIT(&rx_refcnt_mutex);
 	call->flags &= ~RX_CALL_WAIT_PACKETS;
 	USERPRI;
     }
@@ -1990,7 +1989,6 @@ rxi_ReceiveDebugPacket(struct rx_packet *ap, osi_socket asocket,
 			tpeer.reSends = htonl(tp->reSends);
 			tpeer.inPacketSkew = htonl(tp->inPacketSkew);
 			tpeer.outPacketSkew = htonl(tp->outPacketSkew);
-			tpeer.rateFlag = htonl(tp->rateFlag);
 			tpeer.natMTU = htons(tp->natMTU);
 			tpeer.maxMTU = htons(tp->maxMTU);
 			tpeer.maxDgramPackets = htons(tp->maxDgramPackets);
