@@ -82,6 +82,7 @@ nn * We are using getopt since we want it to be possible to link to
 #include <rx/rx.h>
 #include <rx/rx_null.h>
 #include <rx/rx_globals.h>
+#include <rx/rx_packet.h>
 
 #ifdef AFS_PTHREAD_ENV
 #include <pthread.h>
@@ -897,7 +898,7 @@ do_client(const char *server, short port, char *filename, afs_int32 command,
 
     if (dumpstats) {
 	rx_PrintStats(stdout);
-	rx_PrintPeerStats(stdout, conn->peer);
+	rx_PrintPeerStats(stdout, rx_PeerOf(conn));
     }
     rx_Finalize();
 
